<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) exit;

class Rayanpay extends PaymentModule
{

	private $_html = '';
	private $_postErrors = array();

	public function __construct()
	{
		$this->name = 'rayanpay';
		$this->tab = 'payments_gateways';
		$this->version = '2.5';
		$this->author = 'https://rayanpay.com';
		$this->currencies = true;
  		$this->currencies_mode = 'checkbox';
  		$this->controllers = array('request', 'verify');
		parent::__construct();
		$this->page = basename(__FILE__, '.php');
		$this->displayName = 'درگاه پرداخت رایان پی';
		$this->description = 'پرداخت از طریق درگاه پرداخت رایان پی';
		$this->confirmUninstall = 'آیا مطمئن هستید می خواهید جزئیات را حذف کنید ؟';
		$this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
		if (!sizeof(Currency::checkPaymentCurrencies($this->id)))
			$this->warning = 'لطفا واحد پولی درگاه را تنظیم نمایید';
		$config = Configuration::getMultiple(array('rayanpay_merchantid', 'rayanpay_title', 'rayanpay_rate', ''));
		if (!isset($config['rayanpay_merchantid'],$config['rayanpay_title'],$config['rayanpay_rate']))
			$this->warning = 'لطفا اطلاعات پذیرندگی درگاه را تنظیم نمایید';
	}

	public function install()
	{
		if (!parent::install() or !Configuration::updateValue('rayanpay_merchantid', '') or !Configuration::updateValue('rayanpay_title', '') or !Configuration::updateValue('rayanpay_rate', '') or !$this->registerHook('paymentOptions') or !$this->registerHook('paymentReturn'))
		return false;
		return true;
	}

	public function uninstall()
	{
		if (!Configuration::deleteByName('rayanpay_merchantid') or !Configuration::deleteByName('rayanpay_title') or !Configuration::deleteByName('rayanpay_rate') or !parent::uninstall())
		return false;
		return true;
	}

	public function displayFormSettings()
	{
		$this->_html .= '<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
			<fieldset>
				<legend>تنظیمات</legend>
				<label> عنوان نمایشی درگاه رایان پی </label>
				<div class="margin-form"><input type="text" size="30" name="rayanpay_title" id="rayanpay_title" value="'.Configuration::get('rayanpay_title').'" /></div>
				<label> کد پذیرنده درگاه پرداخت رایان پی </label>
				<div class="margin-form"><input type="text" size="30" name="rayanpay_merchantid" id="rayanpay_merchantid" value="'.Configuration::get('rayanpay_merchantid').'" /></div>
				<label> نرخ ارز </label>
				<div class="margin-form"><input type="number" min="0.01" step="0.01" size="30" name="rayanpay_rate" id="rayanpay_rate" value="'.(Configuration::get('rayanpay_rate')?:1).'" /></div>
				<center><input type="submit" name="submit" value="ذخیره" class="button" /></center>
			</fieldset>
		</form>';
	}

	public function displayConf()
	{
		$this->_html .= '<div class="conf confirm">تنظیمات بروز رسانی شد</div>';
	}

	public function displayErrors()
	{
		foreach ($this->_postErrors as $err)
		$this->_html .= '<div class="alert error">'. $err .'</div>';
	}

	public function getContent()
	{
		$this->displayRayanPay();
		$this->postprocess();
		$this->displayFormSettings();
		return $this->_html;
	}
	
	private function postprocess()
	{
		if (isset($_POST['submit']))
		{
			if (empty($_POST['rayanpay_merchantid']))
				$this->_postErrors[] = 'ورود کد پذیرنده درگاه رایان پی الزامیست.';
			if (empty($_POST['rayanpay_title']))
				$this->_postErrors[] = 'ورود نام نمایشی درگاه رایان پی الزامیست.';
			if (empty($_POST['rayanpay_rate']))
				$this->_postErrors[] = 'ورود نرخ ارز درگاه رایان پی الزامیست.';
			if (!sizeof($this->_postErrors))
			{
				Configuration::updateValue('rayanpay_merchantid', $_POST['rayanpay_merchantid']);
				Configuration::updateValue('rayanpay_title', $_POST['rayanpay_title']);
				Configuration::updateValue('rayanpay_rate', $_POST['rayanpay_rate']);
				return $this->displayConf();
			}
			return $this->displayErrors();
		}
	}

	private function displayRayanPay()
	{
		$this->_html .= '<h2>درگاه پرداخت رایان پی</h2><img src="../modules/rayanpay/logo.png" style="float:left; margin-right:15px;"><b>این افزونه امکان پرداخت از طریق درگاه پرداخت رایان پی را برای شما فراهم می کند.</b><br /><br />';
	}

	public function execPayment($cart)
	{
		global $cookie, $smarty;
		$amount = $cart->getOrderTotal(true, Cart::BOTH);
		$rate = Configuration::get('rayanpay_rate')?:1;
		if ($rate > 0) $amount *= $rate;
		$address  = new Address(intval($cart->id_address_invoice));
		try {
			$c = new SoapClient('https://pms.rayanpay.com/pg/services/webgate/wsdl');
			$r = $c->PaymentRequest($p = array(
					'MerchantId'  => Configuration::get('rayanpay_merchantid'),
					'Amount'      => intval(ceil($amount)),
					'CallbackURL' => $this->context->link->getModuleLink($this->name, 'verify', array('ResNum' => $cart->id), true)
				));
			if ($r->PaymentRequestResult->Status == 100)
			{
				$u = 'https://pms.rayanpay.com/pg/startpay/' . $r->PaymentRequestResult->Authority;
				@header('location: ' . $u);
				Tools::redirectLink($u);
				return '<p>در حال اتصال به درگاه پرداخت رایان پی ...</p><script>document.location="'.$u.'";</script>';
				
			}
			else
			{
				throw new exception($this->getRayanPayResponseStatus($r->PaymentRequestResult->Status));
			}
		} catch (exception $e) {
			return 'خطا : ' . $e->getMessage();
		}
	}

	public function confirmPayment($amount)
	{
		if (strtoupper($_REQUEST['Status']) == 'OK')
		{
			try {
				$rate = Configuration::get('rayanpay_rate')?:1;
				if ($rate > 0) $amount *= $rate;
				$c = new SoapClient('https://pms.rayanpay.com/pg/services/webgate/wsdl');
				$r = $c->PaymentVerification($p = array(
						'MerchantId' => Configuration::get('rayanpay_merchantid'),
						'Amount'     => intval(ceil($amount)),
						'Authority'  => $_REQUEST['Authority']
					));
				if ($r->PaymentVerificationResult->Status == 100)
				{
					return array(true, $r->PaymentVerificationResult->RefID);
				}
				else
				{
					return array(false, 'خطای استعلام پرداخت : ' . $this->getRayanPayResponseStatus($r->PaymentVerificationResult->Status));
				}
			} catch (exception $e) {
				echo $e->getMessage();
				exit;
			}

		}
		else
		{
			return array(false, 'خطای استعلام پرداخت : ' . $this->getRayanPayResponseStatus(strtoupper($_REQUEST['Status'])));
		}
	}

	public function showMessages($result)
	{
		$this->_postErrors[] = $result;
		$this->displayErrors();
		echo $this->_html;
		return $result;
	}

	public function hookPaymentOptions($params)
	{
		if (!$this->active) return;
		//if (!$this->checkCurrency($params['cart'])) return;
		$cart = $this->context->cart;
		$total = Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH));
		$view = '<section><p class="payment_module"><img src="modules/rayanpay/logo.png" alt="'.Configuration::get('rayanpay_title').'" />'.Configuration::get('rayanpay_title').' <br /></p></section>';
		$newOption = new PaymentOption();
		$newOption->setModuleName($this->name)->setCallToActionText(Configuration::get('rayanpay_title'))->setAction($this->context->link->getModuleLink($this->name, 'request', array(), true))->setAdditionalInformation($view);
		return array($newOption);
	}

	public function getRayanPayResponseStatus($code)
	{
		switch($code)
		{
			case -1:   return 'اطلاعات ارسال شده ناقص است';
			case -2:   return 'آی پی یا کد پذیرنده پذیرنده صحیح نیست';
			case -3:   return 'با توجه به محدودیت های شاپرك امكان پرداخت با رقم درخواست شده میسر نمی باشد';
			case -11:  return 'درخواست مورد نظر یافت نشد';
			case -21:  return 'هیچ نوع عملیات مالی برای این تراكنش یافت نشد';
			case -22:  return 'تراكنش نا موفق می باشد';
			case -33:  return 'رقم تراكنش با رقم پرداخت شده مطابقت ندارد';
			case -40:  return 'اجازه دسترسی به متد مربوطه وجود ندارد';
			case -41:  return 'اطلاعات ارسال شده مربوط به توضیحات دیتا غیرمعتبر میباشد';
			case -100: return 'تراکنش در انتظار پرداخت';
			case -101: return 'آدرس بازگشت مشتری خالی است';
			case -102: return 'در پرداخت خطایی رخ داده است';
			case -103: return 'وضعیت پرداخت جهت تایید نادرست است';
			case -104: return 'فروشگاهی با شناسه ارسالی یافت نشد';
			case -105: return 'شناسه مرجع تراکنش اشتباه است';
			case -106: return 'خطای تایید پرداخت';
			case -107: return 'وضعیت پرداخت صحیح نیست';
			case -109: return 'فروشگاه غیر فعال است';
			case -110: return 'شناسه ارسال شده نامعتبر است';
			case -111: return 'پرداخت با شناسه ارسالی یافت نشد';
			case -112: return 'فرمت توضیحات اشتباه است';
			case -113: return 'فرمت موبایل اشتباه است.';
			case 100:  return 'تراکنش با موفقیت انجام شد';
			case 'NOK': return 'پرداخت از سوی کاربر لغو شد';
		}
		return 'خطای غیر منتظره . کد '.$code;
	}

}
